#include "Header.hh"
#include "Block.hh"
#include "Utils.hh"
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <vector>

using std::string;
using namespace Header;
 
namespace Block
{

  DataBlock::DataBlock(SURVEY inputSurveyType)
  {
    headerString = string(" ",headerLength);
    Header::SurveySchema *temp = new Header::SurveySchema(inputSurveyType);
    this->survey = *temp;
    this->surveyType = inputSurveyType;
    this->data = new char[dataLength];
//     this->surveyName = survey;
    delete temp;
  }
  //--------------------------------------------------------------------------

  DataBlock::~DataBlock()
  {
    delete [] data;
  }
  //--------------------------------------------------------------------------

  bool DataBlock::readBlock(std::ifstream &file)
  {
    if(!this->readHeader(file)) return false;
    this->readData(file);
    return true;
  }
  //--------------------------------------------------------------------------

  bool DataBlock::readHeader(std::ifstream &file)
  {
    char *hdr = new char[headerLength];
    file.read( hdr, headerLength*sizeof(char) );
    if(file.eof()) return false;
    for(int c=0;c<headerLength;c++) this->headerString[c] = hdr[c];
    return true;
  }
  //--------------------------------------------------------------------------

  void DataBlock::readData(std::ifstream &file)
  {
    file.read( data, dataLength*sizeof(char) );
  }
  //--------------------------------------------------------------------------

  bool DataBlock::telescopeOK()
  {
    std::string move=this->getKeyword<std::string>(MOVE);

    if(this->surveyType == S70){
      if(move == "01") return true;
      else return false;
    }
    else if((this->surveyType == PM)||(this->surveyType == PH)){
      if(move[1]=='0') return true;
      else return false;
    }
  }
  //--------------------------------------------------------------------------

  std::string DataBlock::getFITSname()
  {
    std::string date = this->getKeyword<std::string>(DATE);
    std::string newdate;
    int j=0;
    for(int i=0;i<date.size();i++)
      if(date[i]!='/') newdate += date[i];

    std::string time = this->getKeyword<std::string>(UT_STRT);
    time = removeLeadingBlanks(time);
    time = decToDMS( dmsToDec(time), 0, "");
    
    std::stringstream output;
    if(this->surveyType == S70) output << "S70_";
    else if(this->surveyType == PM) output << "PM_";
    else if(this->surveyType == PH) output << "PH_";
    else std::cerr << "Error <DataBlock::getFITSname> : unknown survey!\n";

    output << newdate << "_" << time;

    if((this->surveyType == PM)||(this->surveyType == PH))
      output << "_" 
	     << std::setw(2) << std::setfill('0') 
	     << this->getKeyword<int>(IBEAM);

    output << ".fits";

    return output.str();

  }
  //--------------------------------------------------------------------------
//   template <typename Type> 
//   Type DataBlock::getKeyword(int field, int element)
//   {
//     Type output = 
//       this->survey.readParameter<Type>(this->headerString,field,element);
//     return output;
//   }
//   template int DataBlock::getKeyword<int>(int field, int element);
//   template float DataBlock::getKeyword<float>(int field, int element);
//   template string DataBlock::getKeyword<string>(int field, int element);
  //--------------------------------------------------------------------------

  template <class T> 
  T DataBlock::getKeyword(int field, int elementNumber)
  {
    int dim = this->survey.getFieldDimension(field);
    std::stringstream ss;
    std::vector<T> output(dim);
    for(int i=0;i<dim;i++){
      int start = this->survey.getStartPosition(field)+i*dim;
      int length = this->survey.getFieldLength(field);
      ss.str(this->headerString.substr(start,length));
      ss >> output[i];      
    }
    return output[elementNumber];
  }
  template int DataBlock::getKeyword<int>(int field, int elementNumber);
  template float DataBlock::getKeyword<float>(int field, int elementNumber);
  template double DataBlock::getKeyword<double>(int field, int elementNumber);
  template std::string DataBlock::getKeyword<std::string>(int field, int elementNumber);
  
//   template<> int DataBlock::getKeyword<int>(int field, int element){
//     return this->survey.readParameter<int>(this->headerString,field,element);
//   }
//   template<> float DataBlock::getKeyword<float>(int field, int element){
//     return this->survey.readParameter<float>(this->headerString,field,element);
//   }
//   template<> double DataBlock::getKeyword<double>(int field, int element){
//     return this->survey.readParameter<double>(this->headerString,field,element);
//   }
//   template<> string DataBlock::getKeyword<string>(int field, int element){
//    return this->survey.readParameter<string>(this->headerString,field,element);
//   }
  //--------------------------------------------------------------------------
  template <class T> 
  std::vector<T> DataBlock::getFullKeyword(int field)
  {
    int dim = this->survey.getFieldDimension(field);
    std::stringstream ss;
    std::vector<T> output(dim);
    for(int i=0;i<dim;i++){
      int start = this->survey.getStartPosition(field)+i*dim;
      int length = this->survey.getFieldLength(field);
      ss.str(this->headerString.substr(start,length));
      ss >> output[i];      
    }
    return output;
  }
  template<> std::vector<int> DataBlock::getFullKeyword<int>(int field){
    std::vector<int> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]=this->survey.readParameter<int>(this->headerString,field,i);
    return output;
  }
  template<> std::vector<float> DataBlock::getFullKeyword<float>(int field){
    std::vector<float> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]= this->survey.readParameter<float>(this->headerString,field,i);
    return output;
  }
  template<> std::vector<string> DataBlock::getFullKeyword<string>(int field){
    std::vector<string> output(this->survey.getFieldDimension(field));
    for(int i=0;i<this->survey.getFieldDimension(field);i++)
      output[i]=this->survey.readParameter<string>(this->headerString,field,i);
    return output;
  }

}
