#ifndef BLOCK_HH_
#define BLOCK_HH_
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "Header.hh"

namespace Block
{
  const int headerLength = 640;
  const int dataLength = 49152; // = 48*1024

  class DataBlock
  {
  public:
    DataBlock(Header::SURVEY surveyType);
    virtual ~DataBlock();
    bool readBlock(std::ifstream &file);
    bool readHeader(std::ifstream &file);
    void readData(std::ifstream &file);
    char getDatum(int i){return data[i];};
    char *getData(){return data;};
    std::string getSurveyName(){return surveyName;};
    Header::SURVEY getSurveyType(){return surveyType;};
    bool telescopeOK();
    std::string getFITSname();
    template <typename T> T getKeyword(int field, int element=0);
    template <typename T> std::vector<T> getFullKeyword(int field);
    Header::SurveySchema& Survey(){
      Header::SurveySchema& rsurv=survey; return rsurv;
    };

  private:
    std::string headerString;    // the header string of the block
    char *data;                  // the set of data
    Header::SURVEY surveyType;   // what type of survey the block deals with
    Header::SurveySchema survey; // how the headers of the survey are defined
    std::string surveyName;      // the name of the survey
  };

}
#endif /*BLOCK_HH_*/
