#ifndef PSRFITS_HH_
#define PSRFITS_HH_
#include <fitsio.h>
#include <string>
#include "Block.hh"
#include "Header.hh"

using namespace Block;
using namespace Header;

namespace FITS
{
	
  enum STATE { OPEN, CLOSED }; // the state of the file
	
  void FITSError(std::string function, int status);
  
  const int SUBINT_kw_nch_strt = 0;
  const std::string SUBINT_kw_int_type = "TIME";
  const std::string SUBINT_kw_int_unit = "SEC";
  const std::string PHU_kw_obs_mode = "SEARCH";
  const std::string PHU_kw_coord_md = "J2000.0";
  
  class PSRFits
  {
  public:
    PSRFits();
    PSRFits(std::string name);
    PSRFits(DataBlock &block);
    virtual ~PSRFits(){};
    bool isNewFile(){return newFile;};
    STATE getFilestate(){return filestate;};
    void setFilestate(STATE S){filestate=S;};
    std::string getUT(){return UT_initial;};
    void setUT(std::string s){UT_initial=s;};
    std::string getDate(){return DATE_initial;};
    void setDate(std::string s){DATE_initial=s;};
    int getBeam(){return beam_number;};
    void setBeam(int i){beam_number=i;};

    int openFile();
    int openFile(DataBlock &block);
    int openFile(std::string name);
    void fixPSRFITS();
    int closeFile();
    int gotoPHU();
    int gotoSubint();
    template <class T> int updateKeyword(std::string name, T value); 
    template <class T> int writeColumnValue(char *name, T value);
    int getColnum(char *name);
    long getNumRows();
    int startNewRow();
    void writePHU(DataBlock &block);
    int  writeSubint(DataBlock &block);
    void startNewColumn();
    void writeColumnValue();
    void writeSupportArrays(DataBlock &block);
    void writeDataArray(DataBlock &block);
    void setFITSname(Header::SURVEY surveyType);

  private:
    fitsfile *fptr;
    std::string FITSname;     // the output FITS filename
    bool newFile;             // is the file a new one?
    STATE filestate;          // OPEN or CLOSED?
    std::string UT_initial;   // the UT at the start of the FITS file data.
    std::string DATE_initial; // the UT date at the start of the data.
    int beam_number;  // what beam are we using -- necessary for FITS file name
  };

}
#endif /*PSRFITS_HH_*/
